@@ -236,7 +236,7 @@ class File:
             sio = io.StringIO(stdin)
             lines, self.nl, mixed, self.sha256 = get_lines(sio)
         elif self.filename is not None and os.path.isfile(self.filename):
-            with open(self.filename, newline='') as f:
+            with open(self.filename, encoding='UTF-8', newline='') as f:
                 lines, self.nl, mixed, self.sha256 = get_lines(f)
         else:
             if self.filename is not None:
@@ -524,20 +524,29 @@ class File:
         assert self.selection.start is not None
         sel_y, sel_x = self.selection.start
         (s_y, _), (e_y, _) = self.selection.get()
+        tab_string = self.buf.tab_string
+        tab_size = len(tab_string)
         for l_y in range(s_y, e_y + 1):
             if self.buf[l_y]:
-                self.buf[l_y] = ' ' * self.buf.tab_size + self.buf[l_y]
+                self.buf[l_y] = tab_string + self.buf[l_y]
                 if l_y == self.buf.y:
-                    self.buf.x += self.buf.tab_size
+                    self.buf.x += tab_size
                 if l_y == sel_y and sel_x != 0:
-                    sel_x += self.buf.tab_size
+                    sel_x += tab_size
         self.selection.set(sel_y, sel_x, self.buf.y, self.buf.x)
 
     @edit_action('insert tab', final=False)
     def _tab(self, margin: Margin) -> None:
-        n = self.buf.tab_size - self.buf.x % self.buf.tab_size
+        tab_string = self.buf.tab_string
+        if tab_string == '\t':
+            n = 1
+        else:
+            n = self.buf.tab_size - self.buf.x % self.buf.tab_size
+            tab_string = tab_string[:n]
         line = self.buf[self.buf.y]
-        self.buf[self.buf.y] = line[:self.buf.x] + n * ' ' + line[self.buf.x:]
+        self.buf[self.buf.y] = (
+            line[:self.buf.x] + tab_string + line[self.buf.x:]
+        )
         self.buf.x += n
         self.buf.restore_eof_invariant()
 
@@ -548,9 +557,9 @@ class File:
             self._tab(margin)
 
     def _dedent_line(self, s: str) -> int:
-        bound = min(len(s), self.buf.tab_size)
+        bound = min(len(s), len(self.buf.tab_string))
         i = 0
-        while i < bound and s[i] == ' ':
+        while i < bound and s[i] in (' ', '\t'):
             i += 1
         return i
 
@@ -673,39 +682,58 @@ class File:
     def _is_commented(self, lineno: int, prefix: str) -> bool:
         return self.buf[lineno].lstrip().startswith(prefix)
 
+    def _indent(self, lineno: int) -> str:
+        ws_match = WS_RE.match(self.buf[lineno])
+        assert ws_match is not None
+        return ws_match[0]
+
+    def _minimum_indent_for_selection(self) -> int:
+        s_y, e_y = self._selection_lines()
+        return min(len(self._indent(lineno)) for lineno in range(s_y, e_y))
+
     def _comment_remove(self, lineno: int, prefix: str) -> None:
         line = self.buf[lineno]
-        ws_match = WS_RE.match(line)
-        assert ws_match is not None
-        ws_len = len(ws_match[0])
-        rest_offset = ws_len + len(prefix)
-        if line.startswith(prefix, ws_len):
-            self.buf[lineno] = f'{ws_match[0]}{line[rest_offset:].lstrip()}'
+        indent = self._indent(lineno)
+        ws_len = len(indent)
 
-    def _comment_add(self, lineno: int, prefix: str) -> None:
+        if line.startswith(f'{prefix} ', ws_len):
+            self.buf[lineno] = f'{indent}{line[ws_len + len(prefix) + 1:]}'
+        elif line.startswith(prefix, ws_len):
+            self.buf[lineno] = f'{indent}{line[ws_len + len(prefix):]}'
+
+        if self.buf.y == lineno and self.buf.x > ws_len:
+            self.buf.x -= len(line) - len(self.buf[lineno])
+
+    def _comment_add(self, lineno: int, prefix: str, s_offset: int) -> None:
         line = self.buf[lineno]
-        ws_match = WS_RE.match(line)
-        assert ws_match is not None
-        ws_len = len(ws_match[0])
-        self.buf[lineno] = f'{ws_match[0]}{prefix} {line[ws_len:]}'
+
+        if not line:
+            self.buf[lineno] = f'{prefix}'
+        else:
+            self.buf[lineno] = f'{line[:s_offset]}{prefix} {line[s_offset:]}'
+
+        if lineno == self.buf.y and self.buf.x > s_offset:
+            self.buf.x += len(self.buf[lineno]) - len(line)
 
     @edit_action('comment', final=True)
     def toggle_comment(self, prefix: str) -> None:
         if self._is_commented(self.buf.y, prefix):
             self._comment_remove(self.buf.y, prefix)
         else:
-            self._comment_add(self.buf.y, prefix)
+            ws_len = len(self._indent(self.buf.y))
+            self._comment_add(self.buf.y, prefix, ws_len)
 
     @edit_action('comment selection', final=True)
     @clear_selection
     def toggle_comment_selection(self, prefix: str) -> None:
         s_y, e_y = self._selection_lines()
         commented = self._is_commented(s_y, prefix)
+        minimum_indent = self._minimum_indent_for_selection()
         for lineno in range(s_y, e_y):
             if commented:
                 self._comment_remove(lineno, prefix)
             else:
-                self._comment_add(lineno, prefix)
+                self._comment_add(lineno, prefix, minimum_indent)
 
     DISPATCH = {
         # movement
