@@ -19,7 +19,8 @@ class History:
         history_dir = xdg_data('history')
         os.makedirs(history_dir, exist_ok=True)
         for filename in os.listdir(history_dir):
-            with open(os.path.join(history_dir, filename)) as f:
+            history_filename = os.path.join(history_dir, filename)
+            with open(history_filename, encoding='UTF-8') as f:
                 self.data[filename] = f.read().splitlines()
                 self._orig_len[filename] = len(self.data[filename])
         try:
@@ -28,5 +29,6 @@ class History:
             for k, v in self.data.items():
                 new_history = v[self._orig_len[k]:]
                 if new_history:
-                    with open(os.path.join(history_dir, k), 'a+') as f:
+                    history_filename = os.path.join(history_dir, k)
+                    with open(history_filename, 'a+', encoding='UTF-8') as f:
                         f.write('\n'.join(new_history) + '\n')
