@@ -38,6 +38,8 @@ EditResult = enum.Enum('EditResult', 'EXIT NEXT PREV OPEN')
 SEQUENCE_KEYNAME = {
     '\x1bOH': b'KEY_HOME',
     '\x1bOF': b'KEY_END',
+    '\x1b[1~': b'KEY_HOME',
+    '\x1b[4~': b'KEY_END',
     '\x1b[1;2A': b'KEY_SR',
     '\x1b[1;2B': b'KEY_SF',
     '\x1b[1;2C': b'KEY_SRIGHT',
@@ -60,6 +62,7 @@ SEQUENCE_KEYNAME = {
     '\x1b[1;6D': b'kLFT6',  # Shift + ^Left
     '\x1b[1;6H': b'kHOM6',  # Shift + ^Home
     '\x1b[1;6F': b'kEND6',  # Shift + ^End
+    '\x1b[~': b'KEY_BTAB',  # Shift + Tab
 }
 KEYNAME_REWRITE = {
     # windows-curses: numeric pad arrow keys
@@ -93,6 +96,7 @@ KEYNAME_REWRITE = {
     b'^?': b'KEY_BACKSPACE',
     # linux, perhaps others
     b'^H': b'KEY_BACKSPACE',  # ^Backspace on my keyboard
+    b'^D': b'KEY_DC',
     b'PADENTER': b'^M',  # Enter on numpad
 }
 
@@ -457,6 +461,14 @@ class Screen:
                     for file in self.files:
                         file.buf.set_tab_size(parsed_tab_size)
                     self.status.update('updated!')
+        elif response.startswith(':expandtabs'):
+            for file in self.files:
+                file.buf.expandtabs = True
+            self.status.update('updated!')
+        elif response.startswith(':noexpandtabs'):
+            for file in self.files:
+                file.buf.expandtabs = False
+            self.status.update('updated!')
         elif response == ':comment' or response.startswith(':comment '):
             _, _, comment = response.partition(' ')
             comment = (comment or '#').strip()
@@ -483,7 +495,7 @@ class Screen:
                 self.file.filename = filename
 
         if os.path.isfile(self.file.filename):
-            with open(self.file.filename, newline='') as f:
+            with open(self.file.filename, encoding='UTF-8', newline='') as f:
                 *_, sha256 = get_lines(f)
         else:
             sha256 = hashlib.sha256(b'').hexdigest()
@@ -496,7 +508,7 @@ class Screen:
             self.status.update('(file changed on disk, not implemented)')
             return PromptResult.CANCELLED
 
-        with open(self.file.filename, 'w', newline='') as f:
+        with open(self.file.filename, 'w', encoding='UTF-8', newline='') as f:
             f.write(contents)
 
         self.file.modified = False
@@ -581,7 +593,10 @@ class Screen:
 
 def _init_screen() -> 'curses._CursesWindow':
     # set the escape delay so curses does not pause waiting for sequences
-    if sys.version_info >= (3, 9):  # pragma: no cover
+    if (
+            sys.version_info >= (3, 9) and
+            hasattr(curses, 'set_escdelay')
+    ):  # pragma: no cover
         curses.set_escdelay(25)
     else:  # pragma: no cover
         os.environ.setdefault('ESCDELAY', '25')
